@@ -120,9 +120,12 @@ class VCSFileStartHook(hook.Hook):
 
     def __call__(self):
         # closure to avoid error on code reloading
-        self.repo.looping_task(self.repo.config.get('check-revision-interval'),
-                               lambda refresh=_refresh_repos, r=self.repo: refresh(r))
-
+        interval = self.repo.config.get('check-revision-interval')
+        if interval < 0:
+            return
+        def refresh_vcsrepos():
+            return _refresh_repos(self.repo)
+        self.repo.looping_task(interval, refresh_vcsrepos)
 
 # internals to be able to create new vcs repo revision using rql queries #######
 
