@@ -10,21 +10,16 @@ from __future__ import with_statement
 __docformat__ = "restructuredtext en"
 
 import urllib2
-from os import umask, makedirs, environ
+from os import umask, makedirs
 from os.path import split, exists, dirname
 from datetime import datetime
+from contextlib import contextmanager
+import time
 
-# mercurial returns strings transcoded to the local encoding, but we want
-# predictable output regardless of the current locale
-orig_encoding = environ.get('HGENCODING')
-environ['HGENCODING'] = 'utf-8'
 from mercurial import (hg, ui, context, util, encoding,
                        commands as hgcmd, phases, obsolete)
 from mercurial.node import nullid, short as short_hex
-if orig_encoding is not None:
-    environ['HGENCODING'] = orig_encoding
-else:
-    del environ['HGENCODING']
+from mercurial import scmutil
 
 from logilab.common import tempattr
 
@@ -48,6 +43,13 @@ def spop(aset, item):
     except KeyError:
         pass
 
+@contextmanager
+def force_mercurial_utf8():
+    old_encoding = encoding.encoding
+    encoding.encoding = 'utf-8'
+    yield
+    encoding.encoding = old_encoding
+
 # mercurial 2.4 at least does not contain any code to get
 # the precursors ...
 def iter_direct_known_precursors(ctx):
@@ -142,123 +144,134 @@ class HGRepository(repo.VCSRepository):
     type = 'hg'
 
     def pull_or_clone_cache(self, url):
+        vui = vcsfile_ui(self)
         if not exists(self.path):
             pdir = dirname(self.path)
             if not exists(pdir):
                 makedirs(pdir)
             self.info('cloning %s into %s', url, self.path)
             try:
-                hgcmd.clone(vcsfile_ui(self), str(url),
+                hgcmd.clone(vui, str(url),
                             dest=self.path, stream='compressed', noupdate=True)
             except (util.Abort, urllib2.URLError), ex:
                 raise bridge.VCSException(self.eid, 'url', str(ex))
         else:
             self.debug('pulling from %s into %s', remove_authinfo(url), self.path)
-            vui = vcsfile_ui(self)
             try:
                 hgcmd.pull(vui, hg.repository(vui, self.path), source=str(url))
             except (util.Abort, urllib2.URLError), ex:
                 raise bridge.VCSException(self.eid, 'path', str(ex))
+        if obsolete._enabled:
+            # Wait 15s after a pull and re-pull the null revision, to make it
+            # less likely we're missing in-flight obsolescence markers.
+            # see #2818315 
+            time.sleep(15)
+            try:
+                hgcmd.pull(vui, hg.repository(vui, self.path), source=str(url), rev=('null',))
+            except (util.Abort, urllib2.URLError), ex:
+                raise bridge.VCSException(self.eid, 'path', str(ex))
 
     def import_content(self, repoentity, commitevery=1000):
         """import content from the mercurial repository"""
         assert self.eid == repoentity.eid
         if not exists(self.path):
             self.pull_or_clone_cache(repoentity.source_url)
-        try:
-            repo = self.hgrepo()
-            # mercurial 2.5
-            if hasattr(repo, 'unfiltered'):
-                repo = repo.unfiltered()
-        except Exception, ex:
-            self.exception('Impossible to open repository %s (%s)', self, ex)
-            return
 
-        session = repoentity._cw
-        knownheads = [(str(r.changeset), r.revision)
-                      for r in repoentity.heads_rset().entities()]
-
-        # detect stripped branches
-        needsstrip = False
-        needsrenumbering = False
-        for cset, rev in knownheads:
-            if cset not in repo:
-                needsstrip = True
-                self.warning('%s: strip detected. %s is unknown in local cache',
-                             self.path, cset)
-        if not needsstrip:
-            incoming = repo.revs("not ::%ls", [cset for cset, _rev in knownheads])
-            if incoming:
-                needsrenumbering = session.execute('Any R WHERE R is Revision, '
-                                                   'R from_repository REPO, '
-                                                   'REPO eid %%(reid)s, R revision IN (%s)'
-                                                   % ','.join([str(rev) for rev in incoming]),
-                                                   {'reid': repoentity.eid})
-            if not needsrenumbering:
-                for cset, rev in knownheads:
-                    if repo[str(cset)].rev() != rev:
-                        needsrenumbering = True
-                        break
-
-        if needsstrip or needsrenumbering:
-            self.warning('%s: strip and renumbering from %s.',
-                         repoentity.dc_title(), self.path)
-            # clean all 'revision' attribute to please unicity
-            # constraint when renumbering
-            session.execute('SET R revision 0-REV WHERE R revision REV, R from_repository REPO, '
-                            'REPO eid %(reid)s', {'reid': repoentity.eid})
-            revs = session.execute('Any R WHERE R is Revision, R from_repository REPO, '
-                                   'REPO eid %(reid)s', {'reid': repoentity.eid})
-            for reventity in revs.entities():
-                if str(reventity.changeset) not in repo:
-                    reventity.cw_delete()
-                else:
-                    rev = repo[str(reventity.changeset)].rev()
-                    reventity.set_attributes(revision=rev)
-
-            if session.execute('Revision R WHERE R revision < 0, R from_repository REPO, '
-                                   'REPO eid %(reid)s', {'reid': repoentity.eid}):
-                session.rollback()
-                self.error('Abort stripping and renumbering of Repository %s: '
-                           'detected remaining Revision with negative revision number',
-                           repoentity.eid)
+        with force_mercurial_utf8():
+            try:
+                repo = self.hgrepo()
+                # mercurial 2.5
+                if hasattr(repo, 'unfiltered'):
+                    repo = repo.unfiltered()
+            except Exception, ex:
+                self.exception('Impossible to open repository %s (%s)', self, ex)
                 return
-            session.commit()
-            session.set_cnxset()
-            session.execute(
-                'DELETE VersionedFile X WHERE X from_repository R, R eid %(r)s, '
-                'NOT VC content_for X', {'r': repoentity.eid})
-            # commit and give a chance to threads waiting for a cnxset
-            # by freeing it then reacquiring it
-            session.commit()
-            session.set_cnxset()
 
-        # update phases and visibility of already known revisions
-        self.update_phases(session, repoentity, repo)
-        self.update_hidden(session, repoentity, repo)
-        # XXX we need to update obsolescence markers without new revision
-        #     involved.
-        knownheads = [str(r.changeset) for r in repoentity.heads_rset().entities()]
-        missing = repo.revs("not ::%ls", knownheads)
-        missing.sort()
-        while missing:
-            for rev in missing[:commitevery]:
-                try:
-                    self.import_revision(repoentity, repo, rev)
-                except Exception:
-                    self.critical('error while importing revision %s of %s',
-                                  rev, self.path, exc_info=True)
-                    raise
+            session = repoentity._cw
+            knownheads = [(str(r.changeset), r.revision)
+                          for r in repoentity.heads_rset().entities()]
+
+            # detect stripped branches
+            needsstrip = False
+            needsrenumbering = False
+            for cset, rev in knownheads:
+                if cset not in repo:
+                    needsstrip = True
+                    self.warning('%s: strip detected. %s is unknown in local cache',
+                                 self.path, cset)
+            if not needsstrip:
+                incoming = repo.revs("not ::%ls", [cset for cset, _rev in knownheads])
+                if incoming:
+                    needsrenumbering = session.execute('Any R WHERE R is Revision, '
+                                                       'R from_repository REPO, '
+                                                       'REPO eid %%(reid)s, R revision IN (%s)'
+                                                       % ','.join([str(rev) for rev in incoming]),
+                                                       {'reid': repoentity.eid})
+                if not needsrenumbering:
+                    for cset, rev in knownheads:
+                        if repo.revs("id(%s)" % str(cset)) != [rev]:
+                            needsrenumbering = True
+                            break
+
+            if needsstrip or needsrenumbering:
+                self.warning('%s: strip and renumbering from %s.',
+                             repoentity.dc_title(), self.path)
+                # clean all 'revision' attribute to please unicity
+                # constraint when renumbering
+                session.execute('SET R revision 0-REV WHERE R revision REV, R from_repository REPO, '
+                                'REPO eid %(reid)s', {'reid': repoentity.eid})
+                revs = session.execute('Any R WHERE R is Revision, R from_repository REPO, '
+                                       'REPO eid %(reid)s', {'reid': repoentity.eid})
+                for reventity in revs.entities():
+                    if str(reventity.changeset) not in repo:
+                        reventity.cw_delete()
+                    else:
+                        rev = repo[str(reventity.changeset)].rev()
+                        reventity.set_attributes(revision=rev)
+
+                if session.execute('Revision R WHERE R revision < 0, R from_repository REPO, '
+                                       'REPO eid %(reid)s', {'reid': repoentity.eid}):
+                    session.rollback()
+                    self.error('Abort stripping and renumbering of Repository %s: '
+                               'detected remaining Revision with negative revision number',
+                               repoentity.eid)
+                    return
+                session.commit()
+                session.set_cnxset()
+                session.execute(
+                    'DELETE VersionedFile X WHERE X from_repository R, R eid %(r)s, '
+                    'NOT VC content_for X', {'r': repoentity.eid})
+                # commit and give a chance to threads waiting for a cnxset
+                # by freeing it then reacquiring it
+                session.commit()
+                session.set_cnxset()
+
+            # update phases and visibility of already known revisions
+            self.update_phases(session, repoentity, repo)
+            self.update_hidden(session, repoentity, repo)
+            # XXX we need to update obsolescence markers without new revision
+            #     involved.
+            knownheads = [str(r.changeset) for r in repoentity.heads_rset().entities()]
+            missing = repo.revs("not ::%ls", knownheads)
+            missing.sort()
+            while missing:
+                for rev in missing[:commitevery]:
+                    try:
+                        self.import_revision(repoentity, repo, rev)
+                    except Exception:
+                        self.critical('error while importing revision %s of %s',
+                                      rev, self.path, exc_info=True)
+                        raise
+                # give a chance to threads waiting for a cnxset by
+                # freeing it then reacquiring it
+                session.commit()
+                session.set_cnxset()
+                del missing[:commitevery] # pop iterated value
+            # if missing was empty we still need to commit phases and visibility changes
             # give a chance to threads waiting for a cnxset by
             # freeing it then reacquiring it
             session.commit()
             session.set_cnxset()
-            del missing[:commitevery] # pop iterated value
-        # if missing was empty we still need to commit phases and visibility changes
-        # give a chance to threads waiting for a cnxset by
-        # freeing it then reacquiring it
-        session.commit()
-        session.set_cnxset()
 
     def update_phases(self, session, repoentity, repo):
         """ update all phases (this potentially affects all revs since
@@ -339,8 +352,8 @@ class HGRepository(repo.VCSRepository):
         date = datetime.fromtimestamp(ctx.date()[0])
         #taglist = ctx.tags() #repo.nodetags(node)
         revdata = {'date': date, 'revision': i,
-                   'author': unicode(ctx.user(), 'utf-8', 'replace'),
-                   'description': unicode(ctx.description(), 'utf-8', 'replace'),
+                   'author': unicode(ctx.user(), 'utf-8'),
+                   'description': unicode(ctx.description(), 'utf-8'),
                    'changeset': changeset,
                    'phase': unicode(ctx.phasestr()),
                    'branch': unicode(ctx.branch()),
@@ -529,27 +542,31 @@ class HGTransaction(object):
         # ensure mercurial will use configured repo encoding, not locale's
         # encoding
         # XXX modifying module's global is not very nice but I've no other idea
-        encoding.encoding = repohdlr.encoding
-        ctx = context.memctx(self.repo, (self.hgparent, None), msg,
-                             self.changes.keys(), self._filectx, author,
-                             extra=self.extra)
-        node = self.repo.commitctx(ctx)
-        # update revision's record to set correct changeset
-        #
-        # take care, self.repo != self.session.repo (eg mercurial repo instance
-        # / rql repository)
-        source = self.session.repo.system_source
-        # remove previous cached value to only save the changeset
-        with tempattr(revision, 'cw_edited', EditedEntity(revision)) as rev:
-            rev.cw_edited['changeset'] = unicode(short_hex(node))
-            rev.cw_edited['phase'] = unicode(phases.phasenames[self.repo[node].phase()])
-            source.update_entity(self.session, rev)
-        # we need to explicitly commit the connection to the database since this
-        # method is actually called from a postcommit event, i.e. once cnxset has
-        # already been commited
-        self.session.cnxset.connection('system').commit()
-        # XXX restore entity's dict?
-        self.rollback()
+        old_encoding = encoding.encoding
+        try:
+            encoding.encoding = repohdlr.encoding
+            ctx = context.memctx(self.repo, (self.hgparent, None), msg,
+                                 self.changes.keys(), self._filectx, author,
+                                 extra=self.extra)
+            node = self.repo.commitctx(ctx)
+            # update revision's record to set correct changeset
+            #
+            # take care, self.repo != self.session.repo (eg mercurial repo
+            # instance / rql repository)
+            source = self.session.repo.system_source
+            # remove previous cached value to only save the changeset
+            with tempattr(revision, 'cw_edited', EditedEntity(revision)) as rev:
+                rev.cw_edited['changeset'] = unicode(short_hex(node))
+                rev.cw_edited['phase'] = unicode(self.repo[node].phasestr())
+                source.update_entity(self.session, rev)
+            # we need to explicitly commit the connection to the database since
+            # this method is actually called from a postcommit event, i.e. once
+            # cnxset has already been commited
+            self.session.cnxset.connection('system').commit()
+            # XXX restore entity's dict?
+            self.rollback()
+        finally:
+            encoding.encoding = old_encoding
 
     def rollback(self):
         try:
