@@ -90,7 +90,11 @@ class Revision(EntityType):
     branch = String(indexed=True, maxsize=255,
                     description=_("branch of this revision."),
                    __permissions__=RO_ATTRPERMS)
-    # XXX tags
+
+    phase = String(description=_('phase of the changeset'), indexed=True,
+                   default='draft', vocabulary=[_('secret'), _('draft'), _('public')])
+
+    hidden = Boolean(required=True, indexed=True)
 
 
 class VersionedFile(EntityType):
@@ -210,6 +214,9 @@ class vc_rename(RelationDefinition):
     subject = object = 'VersionContent'
     cardinality = '?*'
 
+class obsoletes(RelationDefinition):
+    subject = 'Revision'
+    object = 'Revision'
 
 class granted_permission(RelationDefinition):
     subject = 'Repository'
